"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.randomizeFileName = exports.uploadFiles = void 0;
const umi_bundle_defaults_1 = require("@metaplex-foundation/umi-bundle-defaults");
const umi_uploader_irys_1 = require("@metaplex-foundation/umi-uploader-irys");
const web3_js_1 = require("@solana/web3.js");
const umi_1 = require("@metaplex-foundation/umi");
const sleep = (ms) => new Promise((resolve) => setTimeout(resolve, ms));
const umi_signer_wallet_adapters_1 = require("@metaplex-foundation/umi-signer-wallet-adapters");
function uploadFiles(provider, files, delegateWallet, tries = 5) {
    return __awaiter(this, void 0, void 0, function* () {
        if (files.length == 0) {
            return [];
        }
        if (provider) {
            const umi = (0, umi_bundle_defaults_1.createUmi)(provider.connection.rpcEndpoint)
                .use((0, umi_signer_wallet_adapters_1.walletAdapterIdentity)(delegateWallet))
                .use((0, umi_uploader_irys_1.irysUploader)());
            const res = yield withRetries(() => __awaiter(this, void 0, void 0, function* () {
                const uploaded = (yield umi.uploader.upload(files));
                if (uploaded.length !== files.length) {
                    throw new Error("Upload failed");
                }
                return uploaded;
            }), tries);
            return res;
        }
    });
}
exports.uploadFiles = uploadFiles;
function randomizeFileName(file) {
    const ext = file.name.split(".").pop();
    const name = randomIdentifier() + (ext ? `.${ext}` : "");
    Object.defineProperty(file, "name", {
        writable: true,
        value: name,
    });
}
exports.randomizeFileName = randomizeFileName;
function randomIdentifier() {
    return Math.random().toString(32).slice(2);
}
// docusaurus SSR has issues with Keypair.fromSecretKey running, not sure why.
const getDevnetWallet = () => {
    try {
        return web3_js_1.Keypair.fromSecretKey(new Uint8Array([
            17, 83, 103, 136, 230, 98, 37, 214, 218, 31, 168, 218, 184, 30, 163, 18,
            164, 101, 117, 232, 151, 205, 200,
        ]));
    }
    catch (e) {
        //ignore
    }
};
// A devnet wallet loaded with 1 SHDW for testing in devnet. Yes, anyone can mess with this wallet.
// If they do, devnet shdw things will not continue working. That's life. If you find this,
// please don't be an asshole.
const DEVNET_WALLET = getDevnetWallet();
function maybeUseDevnetWallet(connection, delegateWallet) {
    // @ts-ignore
    if (connection._rpcEndpoint.includes("dev") || connection._rpcEndpoint.includes("localhost")) {
        return {
            publicKey: (0, umi_1.publicKey)(DEVNET_WALLET.publicKey.toString()),
            secretKey: DEVNET_WALLET.secretKey,
        };
    }
    return delegateWallet;
}
function withRetries(arg0, tries = 3) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield arg0();
        }
        catch (e) {
            if (tries > 0) {
                console.warn(`Failed tx, retrying up to ${tries} more times.`, e);
                yield sleep(1000);
                return withRetries(arg0, tries - 1);
            }
            throw e;
        }
    });
}
//# sourceMappingURL=shdw.js.map