"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SplTokenMetadata = exports.getImageFromMeta = exports.MetadataCategory = void 0;
const mpl_token_metadata_1 = require("@metaplex-foundation/mpl-token-metadata");
const umi_1 = require("@metaplex-foundation/umi");
const web3_js_1 = require("@solana/web3.js");
const _1 = require(".");
const arweave_1 = require("./arweave");
// @ts-ignore
const localstorage_memory_1 = __importDefault(require("localstorage-memory"));
const shdw_1 = require("./shdw");
const umi_2 = require("@metaplex-foundation/umi");
const umi_bundle_defaults_1 = require("@metaplex-foundation/umi-bundle-defaults");
const umi_signer_wallet_adapters_1 = require("@metaplex-foundation/umi-signer-wallet-adapters");
const web3_js_2 = require("@solana/web3.js");
const web3_js_3 = require("@solana/web3.js");
var MetadataCategory;
(function (MetadataCategory) {
    MetadataCategory["Audio"] = "audio";
    MetadataCategory["Video"] = "video";
    MetadataCategory["Image"] = "image";
    MetadataCategory["VR"] = "vr";
})(MetadataCategory = exports.MetadataCategory || (exports.MetadataCategory = {}));
const USE_CDN = false; // copied from metaplex. Guess support isn't there yet?
const routeCDN = (uri) => {
    let result = uri;
    if (USE_CDN) {
        result = uri.replace("https://arweave.net/", "https://coldcdn.com/api/cdn/bronil/");
    }
    return result;
};
function getImageFromMeta(meta) {
    if (meta.image) {
        return meta.image;
    }
    else {
        const found = (meta.properties.files || []).find((f) => typeof f !== "string" && f.type === "Ima").uri;
        return found;
    }
}
exports.getImageFromMeta = getImageFromMeta;
const imageFromJson = (newUri, extended) => {
    const image = getImageFromMeta(extended);
    if (image) {
        const file = image.startsWith("http")
            ? extended.image
            : `${newUri}/${extended.image}`;
        return routeCDN(file);
    }
};
//@ts-ignore
const localStorage = (typeof global !== "undefined" && global.localStorage) || localstorage_memory_1.default;
class SplTokenMetadata {
    static init(provider) {
        return __awaiter(this, void 0, void 0, function* () {
            return new this({
                provider,
            });
        });
    }
    constructor(opts) {
        this.provider = opts.provider;
    }
    static attributesToRecord(attributes) {
        if (!attributes) {
            return undefined;
        }
        return attributes.reduce((acc, att) => {
            if (att.trait_type)
                acc[att.trait_type] = att.value;
            return acc;
        }, {});
    }
    static getArweaveMetadata(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            if (uri && uri.length > 0) {
                const newUri = routeCDN(uri);
                const cached = localStorage.getItem(newUri);
                if (cached) {
                    return JSON.parse(cached);
                }
                else {
                    try {
                        // TODO: BL handle concurrent calls to avoid double query
                        const result = yield fetch(newUri);
                        let data = yield result.json();
                        if (data.uri) {
                            data = Object.assign(Object.assign({}, data), (yield SplTokenMetadata.getArweaveMetadata(data.uri)));
                        }
                        try {
                            localStorage.setItem(newUri, JSON.stringify(data));
                        }
                        catch (e) {
                            // ignore
                        }
                        return data;
                    }
                    catch (e) {
                        console.log(`Could not fetch from ${uri}`, e);
                        return undefined;
                    }
                }
            }
        });
    }
    static getImage(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            if (uri) {
                const newUri = routeCDN(uri);
                const metadata = yield SplTokenMetadata.getArweaveMetadata(uri);
                // @ts-ignore
                if (metadata.uri) {
                    // @ts-ignore
                    return SplTokenMetadata.getImage(metadata.uri);
                }
                return imageFromJson(newUri, metadata);
            }
        });
    }
    getEditionInfo(metadata) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!metadata) {
                return {};
            }
            return {
                edition: undefined,
                masterEdition: undefined,
            };
        });
    }
    getTokenMetadata(metadataKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `https://mainnet.helius-rpc.com/?api-key=0d4b4fd6-c2fc-4f55-b615-a23bab1ffc85`;
            try {
                const response = yield fetch(url, {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json',
                    },
                    body: JSON.stringify({
                        jsonrpc: '2.0',
                        id: 'my-id',
                        method: 'getAsset',
                        params: {
                            id: metadataKey.toString(),
                        },
                    }),
                });
                const { result } = yield response.json();
                console.log("Asset Data: ", result);
                if (result) {
                    return {
                        displayName: result.content.metadata.name,
                        metadata: result,
                        metadataKey,
                        image: result.content.links.image,
                        mint: (0, umi_1.publicKey)(result.mint),
                        data: result.content.metadata,
                        description: result.content.metadata.description,
                    };
                }
            }
            catch (error) {
                console.error('Error fetching asset:', error);
            }
        });
    }
    sendInstructions(instructions, signers, payer) {
        return (0, _1.sendInstructions)(new Map(), this.provider, instructions, signers, payer);
    }
    uploadMetadata(args, wallet) {
        return __awaiter(this, void 0, void 0, function* () {
            let randomId = Math.floor(Math.random() * (999 - 100 + 1) + 100);
            // Upload image first
            const imageBuffer = yield args.image.arrayBuffer();
            const uint8Array = new Uint8Array(imageBuffer);
            const genericImage = (0, umi_2.createGenericFile)(uint8Array, `${args.image.name}`);
            const imageUrl = yield (0, shdw_1.uploadFiles)(this.provider, [genericImage], wallet);
            // Use the uploaded image URL in the metadata
            const imageUri = imageUrl[0];
            const metadata = Object.assign({ name: args.name, symbol: args.symbol, description: args.description, image: imageUri, attributes: args.attributes, external_url: args.externalUrl || "", animation_rl: args.animationUrl, creators: args.creators ? args.creators : null, seller_fee_basis_points: 0 }, (args.extraMetadata || {}));
            const metadataBuffer = new TextEncoder().encode(JSON.stringify(metadata));
            const genericMetadata = (0, umi_2.createGenericFile)(metadataBuffer, `${args.mint}-${randomId}.json`);
            const urls = yield (0, shdw_1.uploadFiles)(this.provider, [genericMetadata], wallet);
            return urls[0];
        });
    }
    /**
     * Wrapper function that prepays for arweave metadata files in SOL, then uploads them to arweave and returns the url
     *
     * @param args
     * @returns
     */
    createArweaveMetadata(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { txid, files } = yield this.presignCreateArweaveUrl(args);
            let env = args.env;
            if (!env) {
                // @ts-ignore
                const url = this.provider.connection._rpcEndpoint;
                if (url.includes("devnet")) {
                    env = "devnet";
                }
                else {
                    env = "mainnet-beta";
                }
            }
            const uri = yield this.getArweaveUrl({
                txid,
                mint: args.mint,
                files,
                env,
                uploadUrl: args.uploadUrl || arweave_1.ARWEAVE_UPLOAD_URL,
            });
            return uri;
        });
    }
    presignCreateArweaveUrlInstructions({ name, symbol, description = "", image, creators, files = [], payer = this.provider.wallet.publicKey, existingFiles, attributes, externalUrl, animationUrl, extraMetadata, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const metadata = Object.assign({ name,
                symbol,
                description,
                image,
                attributes, externalUrl: externalUrl || "", animationUrl, properties: {
                    category: MetadataCategory.Image,
                    files: [...(existingFiles || []), ...files],
                }, creators: creators ? creators : null, sellerFeeBasisPoints: 0 }, (extraMetadata || {}));
            const realFiles = yield (0, arweave_1.getFilesWithMetadata)(files, metadata);
            const prepayTxnInstructions = yield (0, arweave_1.prePayForFilesInstructions)(payer, realFiles);
            return {
                instructions: prepayTxnInstructions,
                signers: [],
                output: {
                    files: realFiles,
                },
            };
        });
    }
    presignCreateArweaveUrl(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { output: { files }, instructions, signers, } = yield this.presignCreateArweaveUrlInstructions(args);
            const txid = yield this.sendInstructions(instructions, signers);
            return {
                files,
                txid,
            };
        });
    }
    getArweaveUrl({ txid, mint, files = [], uploadUrl = arweave_1.ARWEAVE_UPLOAD_URL, env = "mainnet-beta", }) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, arweave_1.uploadToArweave)(txid, mint, files, uploadUrl, env);
            const metadataFile = result.messages.find((m) => m.filename === "manifest.json");
            if (!metadataFile) {
                throw new Error("Metadata file not found");
            }
            // Use the uploaded arweave files in token metadata
            return `https://arweave.net/${metadataFile.transactionId}`;
        });
    }
    createMasterEditionInstructions({ mint, mintAuthority = this.provider.wallet.publicKey, payer = this.provider.wallet.publicKey, }) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                instructions: [],
                signers: [],
                output: {
                    metadata: undefined,
                }
            };
        });
    }
    createMasterEdition(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { instructions, signers, output } = yield this.createMasterEditionInstructions(args);
            yield this.sendInstructions(instructions, signers, args.payer);
            return output;
        });
    }
    verifyCollectionInstructions({ nftMint, collectionMint, payer = this.provider.wallet.publicKey, }) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                instructions: [],
                signers: [],
                output: {
                    metadata: undefined,
                }
            };
        });
    }
    verifyCollection(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { instructions, signers, output } = yield this.verifyCollectionInstructions(args);
            yield this.sendInstructions(instructions, signers, args.payer);
            return output;
        });
    }
    createMetadataInstructions({ data, authority = this.provider.wallet.publicKey, mint, mintAuthority = this.provider.wallet, payer = this.provider.wallet, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const umi = (0, umi_bundle_defaults_1.createUmi)(this.provider.connection.rpcEndpoint).use((0, umi_signer_wallet_adapters_1.walletAdapterIdentity)(payer));
            const transactionBuilder = (yield (0, mpl_token_metadata_1.createMetadataAccountV3)(umi, {
                data,
                mint: (0, umi_1.publicKey)(mint),
                updateAuthority: (0, umi_1.publicKey)(authority),
                mintAuthority: (mintAuthority),
                isMutable: true,
                collectionDetails: null
            }));
            let compiled = yield transactionBuilder.buildWithLatestBlockhash(umi);
            const metadataPubkey = web3_js_1.PublicKey.findProgramAddressSync([
                Buffer.from("metadata"),
                new web3_js_1.PublicKey("metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s").toBuffer(),
                mint.toBuffer(),
            ], new web3_js_1.PublicKey("metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s"))[0];
            const lookupTableAccounts = [];
            for (const table of compiled.message.addressLookupTables) {
                // @ts-ignore
                lookupTableAccounts.push(yield new Connection(umi.rpc.getEndpoint()).getAddressLookupTable(table.publicKey).value);
            }
            const txMessageDecompiled = web3_js_2.TransactionMessage.decompile(web3_js_3.VersionedMessage.deserialize(compiled.serializedMessage), {
                addressLookupTableAccounts: lookupTableAccounts
            });
            return {
                instructions: txMessageDecompiled.instructions,
                signers: [],
                output: {
                    metadata: metadataPubkey,
                },
            };
        });
    }
    getMetadata(metadataKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `https://mainnet.helius-rpc.com/?api-key=0d4b4fd6-c2fc-4f55-b615-a23bab1ffc85`;
            try {
                const response = yield fetch(url, {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json',
                    },
                    body: JSON.stringify({
                        jsonrpc: '2.0',
                        id: 'my-id',
                        method: 'getAsset',
                        params: {
                            id: metadataKey.toString(),
                        },
                    }),
                });
                const { result } = yield response.json();
                if (result) {
                    return {
                        mint: result.mint,
                        name: result.content.metadata.name,
                        symbol: result.content.metadata.symbol,
                        description: result.content.metadata.description,
                        image: result.content.links.image,
                        decimals: result.token_info.decimals,
                        supply: result.token_info.supply,
                    };
                }
            }
            catch (error) {
                console.error('Error fetching asset:', error);
            }
            return null;
        });
    }
    createMetadata(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { instructions, signers, output } = yield this.createMetadataInstructions(args);
            yield this.sendInstructions(instructions, signers, args.payer);
            return output;
        });
    }
    updateMetadataInstructions({ data, newAuthority, metadata, updateAuthority, }) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                instructions: [],
                signers: [],
                output: {
                    metadata,
                },
            };
        });
    }
    updateMetadata(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { instructions, signers, output } = yield this.updateMetadataInstructions(args);
            yield this.sendInstructions(instructions, signers, args.payer);
            return output;
        });
    }
}
exports.SplTokenMetadata = SplTokenMetadata;
//# sourceMappingURL=splTokenMetadata.js.map